package com.perforce.hws.core.test;

import com.perforce.hws.HwsPluginManager;
import org.junit.rules.TestRule;
import org.junit.runner.Description;
import org.junit.runners.model.Statement;
import spark.Service;

import java.io.File;

/**
 * Created by pallen on 12/08/2016.
 */
public class EmptySparkRule implements TestRule {

	private final int port;
	private final File plugin;

	private Service service;

	public EmptySparkRule(int port, File plugin) {
		this.port = port;
		this.plugin = plugin;
	}

	public Service getService() {
		return service;
	}

	@Override
	public Statement apply(Statement statement, Description description) {
		return new SparkStatement(statement);
	}

	public class SparkStatement extends Statement {

		private final Statement statement;

		public SparkStatement(Statement statement) {
			this.statement = statement;
		}

		@Override
		public void evaluate() throws Throwable {
			// Load Perforce plugin
			HwsPluginManager pluginManager = HwsPluginManager.getInstance();
			pluginManager.loadPlugin(plugin);

			// Start spark service
			service = Service.ignite();
			service.port(port);

			// Load endpoints
			pluginManager.addRoutes("", service);

			// Evaluate JUnit
			statement.evaluate();

			// Cleanup
			service.stop();
		}
	}
}
